function [ Pout, Ugout, Acop, Udcout, Hout, Kvf, Mvf, ymatout, Wmat ] = esmfsynthesis(tvec, af, Coup, inparam, vfparam, const)
% Time-domain speech synthesis. The vocal tract is modeled as a waveguide
% network. The acoustic source is a 2x2 mass model of the
% vocal folds. 
%
% Input arguments:
% tvec: time vector containing the simulation tuime instants
% af: structure array with fields .A and .l containing the area functions and the length functions, respectively, for each waveguide.
% Coup: structure array containing informations about the corresponding waveguide n: 
%                                                                   .child: 2xN matrix whose the first row contains the numbers of the N waveguides which are connected to n, and the second row contains the positions of the connection
%                                                                   .parents: 2xN matrix whose the first row contains the numbers of the N waveguides to which n is connected, and the second row contains the positions of the connection                                                                   
%                                                                   .twin: indicates the twin waveguides of n
%                                                                   .anabranch: indicates the anastomozing waveguides of n
%                                                                   .rad: indicates whether n radiates or not (1 for radiation, 0 for closed termination)
%                                                                   .Nat: indicates whether n is the glottal chink or not ('Chink' if yes, [] if not)  
% inparam: structure containing input parameters for the glottal source:
%                                                                   .Psub: vector containing the subglottal pressure value (in Pa) at each simulation time instant   
%                                                                   .Ag0: vector containing the abduction value (in m^2) at each simulation time instant
%                                                                   .AgP: vector containing the oscillation amplitude at each simulation time instant
%                                                                   .fo: vector containing the fundamental frequency (in Hz) at each simulation time instant
% vfparam: structure containing the VF properties:
%                                               .Ki, .Mi, .Ri, .Kci: matrix containing the nominal values of the stiffness, mass, damping and coupling stiffness of the 2x2 mass, respectively
%                                               .lgi: nominal length of the vocal folds
%                                               .lvf: width of the vocal folds
%                                               .y0: initial positions of the vocal folds along y (normal to the flow direction)
%                                               .xi: initial positions of the vocal folds along x (flow direction)
%                                               .Hi: initial opening of the glottal constriction
% const: structure containing constant parameters used for the simulation
%
% Ouput arguments:
% Pout: output acoustic pressure
% Ugout: volume velocity through the oscillating part of the vocal folds
% Acop: structure array with fields .P and .U containing respectively the acoustic pressure and the volume velocity inside each tubelet of the each waveguide
% Udcout: matrix containing the low-frequency component of the acoustic flow (first row), and the reynolds number (second row)
% Hout: matrix whose each line contains the glottal opening along the x direction
% Kvf: matrix containing the output stiffness values of the vocal folds
% Mvf: matrix containing the output mass values of the vocal folds
% ymatout: matrix containing the output positions values of the vocal folds
%
% B. Elie, march 2016, last modification 2018. New features :
% - subglottal tract
% - self-oscilalting model of the tongue tip
% - Ishizaka and Flanagan model of the vocal folds

if nargin < 6; const = []; end
if ~isfield(const,'kctc'); const.kctc = 4; end
if ~isfield(const,'rctc'); const.rctc = 2.2; end
if ~isfield(const,'kctctt'); const.kctctt = 1.5; end
if ~isfield(const,'rctctt'); const.rctctt = 1.2; end
if ~isfield(const,'c'); const.c = 352.3313; end
if ~isfield(const,'mu'); const.mu = 1.8500e-05; end
if ~isfield(const,'rho'); const.rho = 1.1255; end
if ~isfield(const,'blim'); const.blim = 1e-5; end
if ~isfield(const,'filt'); const.filt = 'fir1'; end
if ~isfield(const,'sep'); const.sep = 1.2; end
if ~isfield(const,'Rec'); const.Rec = 1.7e3; end
if ~isfield(const,'nmass'); const.nmass = 2; end
if ~isfield(const,'dist_source'); const.dist_source = 2; end
if ~isfield(const,'fh'); const.fh = 200; end
if ~isfield(const,'fh'); const.fl = 4e3; end
if ~isfield(const,'ordh'); const.ordh = 1; end
if ~isfield(const,'ordl'); const.ordl = 7; end
if ~isfield(const,'bf'); const.bf = [0.9000 0.1000 -0.8000]; end
if ~isfield(const,'af'); const.af = 1; end
if ~isfield(const,'K_b'); const.K_b = 1.5982; end
if ~isfield(const,'G'); const.G = 0.0703; end
if ~isfield(const,'S'); const.S = 0.3750; end
if ~isfield(const,'dispcomp'); const.dispcomp = 1; end
if ~isfield(const,'wallyield'); const.wallyield = 1; end
if ~isfield(const,'dynterm'); const.dynterm = 1; end
if ~isfield(const,'beta'); const.beta = 5e-8; end
if ~isfield(const,'Qr'); const.Qr = 0.2; end
if ~isfield(const,'Qrc'); const.Qrc = 0.05; end
if ~isfield(const,'wr'); const.wr = 800; end
if ~isfield(const,'wm'); const.wm = 21; end
if ~isfield(const,'wc'); const.wc = 8450000; end
if ~isfield(const,'selfoscil'); const.selfoscil = 1; end
if ~isfield(const,'amin'); const.amin = 1e-11; end
if ~isfield(const,'model'); const.model = 'ishi'; end
if ~isfield(const,'nmass'); const.nmass = 2; end
if ~isfield(const,'osc_tg'); const.osc_tg = 0; end
if (~isfield(const,'model_tt') && const.osc_tg); const.model_tt = 'smooth'; end
if ~isfield(const,'unst'); const.unst = 1; end
if ~isfield(const,'rmlt'); const.rmlt = 4; end

if (~const.selfoscil) && (~isfield(inparam,'ag') || isempty(inparam(1).ag) || length(inparam(1).ag) < length(tvec))
    warning on; warning('Glottal opneing parameters not set properly for parametric glottis based simulation. Use self-oscillations instead'); warning off;
    const.selfoscil = 1;
end

if (~strcmp(const.model,'smooth') && ~strcmp(const.model,'ishi'))
    warning on;warning('Incorrect definition of the glottis model. Set to smooth contours by default'); warning off;
    const.selfmod = 'smooth';
end

if (const.osc_tg && length(vfparam) < 2)
    warning on; warning('Tongue oscillation is set to 1 but parameters not defined. Tongue oscilations wont be taken into account.'); warning off;
    const.osc_tg = 0;
end

%%%%%%%%%%%%% Check potential erros
%%%%% Zeros 
inparam(1).fo(inparam(1).fo<=60) = 60;

armin = const.amin;
rho = const.rho;
co_mu = const.mu;


%%%%%%%%%%% Definition of all the constants used during resolution.
T = diff(tvec(1:2));
N = length(tvec);
if length(inparam(1).Psub) ~= N; inparam(1).Psub = mean(inparam(1).Psub)*ones(size(tvec)); end
if length(inparam(1).fo) ~= N; inparam(1).fo = 100*ones(size(tvec)); end

%%%%%%%%%% Initialization of glottal parameters
if const.selfoscil    
    yt = vfparam(1).y0; % positions at instant t
    ytm1 = vfparam(1).y0; % positions at t-1
    ytm2 = vfparam(1).y0; % positions at t-2
% 	dgl = vfparam(1).xi(3)-vfparam(1).xi(2);
    xvec = vfparam(1).xi;
    const.lgi = vfparam(1).lgi;
    fovfvec = inparam(1).fo;
end

fs = round(1/T); % Simulation frequency
const.T = T;
Fric_Amp = const.namp;
fny = fs/2; % Nyquist frequency

%%%%%%%%%%% Initialization of the solutions
Npipe = length(af);
Nx = zeros(Npipe,1);
Nrad = sum(arrayfun(@(x) sum(x.rad), Coup));
krad = 1; posrad = zeros(Nrad,1);
Udcvec = zeros(N,1);
Wmat = 0;
agovf = inparam(1).Ag0;

if const.osc_tg 
    %%%%%%%%% Tongue tip
    ytt = vfparam(2).y0;
    yttm1 = zeros(2); % positions at t-1
    yttm2 = zeros(2); % positions at t-2
%     dtt = vfparam(2).xi(3)-vfparam(2).xi(2);
    ptt = vfparam(2).pos;
    utt = 0;
    uttp1 = 0;
    Qtt = 0;
    Qttp1 = 0;
    Uttim1 = 0;
    Hout = NaN(5,N,2); ymatout = NaN(2,2,N,2); Udcout = NaN(3,N); Kvf = ymatout; Mvf = Kvf;
    Qam2tt = vfparam(2).Mi(1,2)/vfparam(2).Mi(1,1);
    agott = inparam(2).Ag0;
    fottvec = inparam(2).fo;
else
    Hout = NaN(5,N); ymatout = NaN(2,2,N); Udcout = NaN(3,N); Kvf = ymatout; Mvf = Kvf; blim = const.blim;
end

chbool = 0; sgbool = 0;
Acop = struct('U',cell(Npipe,1),'P',cell(Npipe,1));
Qp = struct('Vc',cell(Npipe,1),'Q',cell(Npipe,1),'V',cell(Npipe,1),'Qwl',cell(Npipe,1),'Qwc',cell(Npipe,1),'Qnc',cell(Npipe,1));
wn = struct('W',cell(Npipe,1));

for kpipe = 1:Npipe    
    Nch = 0; 
    Nx(kpipe) =  size(af(kpipe).A,1); % length of each pipe
	[ Qp(kpipe).Vc, Qp(kpipe).Q, Qp(kpipe).V, Qp(kpipe).Qwl, Qp(kpipe).Qwc, Qp(kpipe).Qnc ] = Init_Qp(Nx(kpipe));
    child = Coup(kpipe).child; 
    if ~isempty(child); Nch = size(child,2); end    
    Qp(kpipe).Qnc = zeros(1,Nch);
    if Coup(kpipe).rad == 1; posrad(krad) = kpipe; krad = krad+1; end
    if strcmp(Coup(kpipe).Nat,'Chink')        
        Acop(kpipe) = struct('U',zeros(1,N),'P', zeros(1,N)); 
        chbool = 1;        
    else
        Acop(kpipe) = struct('U',  NaN(Nx(kpipe)+1,N), 'P', NaN(Nx(kpipe)+1,N)); 
    end 
    if strcmp(Coup(kpipe).Nat,'subglott')    
        isg = kpipe;
        sgbool = 1;
    end
    wn(kpipe).W = randn(Nx(kpipe)+1,N);
    %%%%%%%%% Check for zero values
    af(kpipe).A(af(kpipe).A <= armin) = armin;
    af(kpipe).l(af(kpipe).l <= armin) = armin;    
end

Ug = zeros(N,1); Pm = Ug;
Qug = 0;
Uk1 = 0;
Ugim1 = 0;
Ach = 0;
lchk = zeros(size(tvec));

if chbool 
    inparam(1).lch(inparam(1).lch<=armin) = armin; Uch = Ug; lchk = vfparam(1).lgi*inparam(1).lch; 
    if length(inparam(1).lch) ~= N; inparam(1).lch = armin*ones(size(tvec)); end
end
if ~chbool; inparam(1).lch = zeros(N,1); end
kwb = 1;
[ac,ic] = min(af(1).A);

if any(ic>=38)
    if strcmp(const.filt,'fir1'); hld = fir1(const.ordl*2,1000/fny,'low'); hld2 = fir1(1,1000/fny,'high'); end
    if strcmp(const.filt,'firls'); hld = firls(const.ordl+const.ordh,[0 6000/fny 10000/fny 1],[0.9 1 1 0]); end 
end
%%%%%%%%%%%%%%%% Resolution of the system at every
warning off
if sgbool; pstmp = 0; h0ivec = powtr(af(isg).A(end,:),1.8, 1.2, 0 ); end
if ~sgbool; h0ivec = vfparam(1).Hi(1)*ones(N,1); end
lgvec = vfparam(1).lgi-lchk;
if ~const.selfoscil
    lc = zeros(size(ac));
    for k = 1:length(ic); lc(k) = af(1).l(ic(k),k); end
    Ag2 = 1./(inparam(1).ag.^2);
    Ac2 = 1./(ac.^2);
    rv_sum = (12*lgvec(:)*vfparam(1).lvf.*Ag2(:).*sqrt(Ag2(:))+8*pi*lc(:).*Ac2(:))*co_mu;
    rk_sum = 1.38*rho*(Ac2(:)+Ag2(:));
    udc = (-rv_sum + sqrt(rv_sum.*rv_sum + 4*rk_sum.*inparam(1).Psub(:)))./(2*rk_sum); 	
    udc(isnan(udc)) = 0;  
    rgvec = 12*co_mu*lgvec.^2*vfparam(1).lvf./(inparam(1).ag.^3);
end

Udcinst = 0;
for k = 1:N
    lg = lgvec(k);
    if (~sgbool) || (k == 1); pstmp = inparam(1).Psub(k); end
    if chbool; Ach = af(end).A(k); end
    af_main = af(1).A(:,k); lf_main = af(1).l(:,k);
    if const.osc_tg
        %%%% Tongue tipKvf
        ltt = powtr( agott(k), 1.8, 1.2, 0 );
        if const.nmass == 2
            h1tti = ytt(2,1)*inparam(2).AgP(k)+ltt;
            h2tti = ytt(2,2)*inparam(2).AgP(k)+ltt;
        else
            h1tti = ytt*inparam(2).AgP(k)+ltt;
            h2tti = ytt*inparam(2).AgP(k)+ltt;
        end
        h0tti = powtr( af(1).A(ptt-1,k), 1.8, 1.2, 0 );
        h3tti = powtr( af(1).A(ptt+1,k), 1.8, 1.2, 0 );
        httvec = [ h0tti, h1tti, h2tti, h3tti ];
        dlt = powtr( ltt, 1.8, 0.2, 1 )*1e2;
        ldtt = af(1).l(vfparam(2).pos,k);
        if strcmp(const.model_tt,'ishi')
            x1tt = ldtt/(1+Qam2tt);  % 1st mass
            x2tt = ldtt-1e-11;   % 2nd mass
            x3tt = ldtt;
        else
            x1tt = 2e-4;
            x2tt = ldtt-2*x1tt;
            x3tt = ldtt;
        end
        xtvec = [ 0 x1tt x2tt x3tt];
        vfparam(2).xi = xtvec;
        [ Rgtt2, ~, ~, hsitt ] = tonguetipZupdate(xtvec, httvec, dlt, const );        
        hsii = hsitt;
        af(1).A(ptt,k) = max(1e-8,powtr(hsii,1.8,1.2,1));        
        if ~isempty(Coup(1).anabranch)
            lat = Coup(1).anabranch;
            cp = Coup(1).child;
            ii = find(cp(1,:) == lat);
            af_main(cp(2,ii(1)):cp(2,ii(2))) = af_main(cp(2,ii(1)):cp(2,ii(2))) + af(lat).A(:,k);
        end
%         [ ack, ick ] = min(af_main);
    end
    
    if const.selfoscil
%         if inparam(1).lch(k) >= 0.9; inparam(1).AgP(k) = 0; end
        h1i = sum(yt(:,1)*inparam(1).AgP(k))+inparam(1).Ag0(k)/lg;
        h2i = sum(yt(:,2)*inparam(1).AgP(k))+inparam(1).Ag0(k)/lg;
        h0i = h0ivec(k);
        h3i = powtr( af(1).A(1,k), 1.8, 1.2, 0 );
        Hvec = [ h0i, h1i, h2i, h3i ];    
        [ Rg2, Rg, Lg, hsi, udct ] = glotZupdate( xvec, Hvec, af_main, lf_main, lg, pstmp, const, Ach );
        
        if ~const.unst; Lg = Lg*0; end
        if strcmp(const.model,'ishi')
            rgv = Rg;
            lgv = Lg;
            Rg = sum(Rg);
            Lg = sum(Lg);
        end
        if const.osc_tg
            Hout(:,k,1) = [ Hvec(:);hsi]; ymatout(:,:,k,1) = yt*inparam(1).AgP(k);
            Hout(:,k,2) = [ httvec(:);hsitt]; ymatout(:,:,k,2) = ytt;
        else            
            Hout(:,k) = [ Hvec(:);hsi]; ymatout(:,:,k) = yt*inparam(1).AgP(k);
        end
        Aglot = max([armin, abs(hsi*lg)]);  
        Udcinst = Udcinst+(udct-Udcinst)*2*pi*500*T;
        Udcvec(k) = Udcinst;
        Udc = udct;       
    else
        Rg2 = 0;
        Lg = 0;
        Rg = rgvec(k);
        Udc = udc(k);
        Aglot = inparam(1).ag(k);
        if const.osc_tg
            Hout(:,k,2) = [ httvec(:);hsitt]; ymatout(:,:,k,2) = ytt;        
        end
    end   

    kpc = 100*((k-1)/(N-1)) ;
    if (kpc >= 10*kwb && const.dispcomp );disp(['Completion: ',round(num2str(kpc)),' %']); kwb = kwb+1; end   
    
    for kpipe = 1:Npipe % Electric components of each pipe    
        if k == 1
            uim1 = zeros(Nx(k),1);
        else
            uim1 = Acop(kpipe).U(:,k-1);
        end
        namp = Fric_Amp; if size(Coup(kpipe).parents,2) > 1; namp = 0; end      
        if kpipe == 1            
            A2o = [ Aglot; af(kpipe).A(:,k) ]; %l2o = [ lg; af(kpipe).l(:,k) ]; fpeak = 0;     
            fpeak = 0;
        else 
            namp = 0;
        end        
        Reyn = Reynold_calc( af_main, Udc, rho, co_mu, 1);
        if (Reyn < const.Rec || strcmp(Coup(kpipe).Nat,'Chink')); namp = 0; end%Rc = Rc*0; end
        if ~strcmp(Coup(kpipe).Nat,'subglott')       
            [ Ljk, Rjk, Cjk, Sradk, Gwk, wlk, wck, Udjk,bjk, Hjk, Rcm, Rcp ] = acou2elec( af(kpipe).l(:,k), af(kpipe).A(:,k), const, uim1,Aglot,0);              
        else            
            [ Ljk, Rjk, Cjk, Sradk, Gwk, wlk, wck, Udjk,bjk, Hjk, Rcm, Rcp ] = acou2elec( af(kpipe).l(:,k), af(kpipe).A(:,k), const, uim1,Aglot,1 );
        end
        if (~const.unst && kpipe == 1 && const.osc_tg)
            Ljk(ptt) = 0;
            Hjk(ptt:ptt+1) = -2*(Ljk(ptt-1:ptt)+Ljk(ptt:ptt+1))/T-Rjk(ptt-1:ptt)-Rjk(ptt:ptt+1)-bjk(ptt-1:ptt)-bjk(ptt:ptt+1)-Rcm(ptt-1:ptt)-Rcp(ptt:ptt+1);
        end
        if strcmp(Coup(kpipe).Nat,'Chink'); Ljk = rho*af(kpipe).l(:,k)./af(kpipe).A(:,k); Rjk = 12*co_mu*(inparam(1).lch(k)*vfparam(1).lgi)^2*af(kpipe).l(:,k)./(af(kpipe).A(:,k).^3); end
%         if strcmp(Coup(kpipe).Nat,'Chink'); Ljk = Ljk*2; Rjk = Rjk*2; end
        W = wn(kpipe).W;
        if namp~=0
            dk = 20;
            bb = max(1,k-dk):min(N,k+dk);
            if ic(k) >= 38 %%%%%% Frication noise generated at the lips (labiodental constriction)
                hl = hld;                 
            else
                fpeak = min(0.3*sqrt(pi)*Udc/(ac(k)*sqrt(ac(k))),fny*0.999);%*const.rho/const.mu;
                ordl = 9+2*(38-ic(k)-1);
                if strcmp(const.filt,'fir1'); hl = fir1(ordl,fpeak/fny,'low'); end
                if strcmp(const.filt,'firls'); hl = firls(ordl+1,[0 fpeak/fny 10000/fny 1],[0.7 1 0 0]); end
            end            
            Wb = W(:,bb); 
            Wb = filter(hl,1,Wb.');
            if ic(k) >= 38; Wb = filter(hld2,1,Wb); end
            Wb = Wb/std(Wb(:));
            phik = namp*(Reyn^2-const.Rec^2);
            Nsk = phik*(Wb(round(length(bb)/2),:).')./(A2o.^2.5)*Udc^3;
            if (kpipe == 1 && const.dynterm); Udjk(ic(k)+1) = const.beta*phik; end
            Nsk(1) = 0;
        else
            Nsk = zeros(size(W,1),1);
        end        
        atermk = af(kpipe).A(end,k);
        Cp(kpipe) = struct('Lj',Ljk,'Rj',Rjk,'Cj',Cjk,'Srad',Sradk,'Gw',Gwk,'wl',wlk,'wc',wck,'Udj',Udjk,'bj',bjk,'Hj',Hjk,'Ns',Nsk,'aterm',atermk,'Rcm',Rcm,'Rcp',Rcp);      
    end
    
    if const.selfoscil
        [ mvfo, kvfo, rvfo, kco ] = VFparamupdate(fovfvec(k), vfparam(1), max(0.1,1-lchk(k)));
        [ ke, re ] = contactparam(mvfo, kvfo, rvfo, Hvec, const,'vf');
        bool = (Hvec(2) <= const.blim) || (Hvec(3) <= const.blim);       

        if const.osc_tg
            Mvf(:,:,k,1) = mvfo; Kvf(:,:,k,1) = kvfo;
            [ mvfott, kvfott, rvfott, kcott ] = VFparamupdate(fottvec(k), vfparam(2), 1);
            Mvf(:,:,k,2) = mvfott; Kvf(:,:,k,2) = kvfott;
            [ kett, rett ] = contactparam(mvfott, kvfott, rvfott, httvec,const, 'tt');
            booltt = (httvec(2) <= 0) || (httvec(3) <= 0);
        else
            Mvf(:,:,k) = mvfo; Kvf(:,:,k) = kvfo;
        end
    else
        bool = (inparam(1).ag<= armin);
        if const.osc_tg           
            [ mvfott, kvfott, rvfott, kcott ] = VFparamupdate(fottvec(k), vfparam(2), 1);
            [ kett, rett ] = contactparam(mvfott, kvfott, rvfott, httvec,const, 'tt');
            booltt = (httvec(2) <= 0) || (httvec(3) <= 0);
        end
    end
    
    pcoef(1) = Rg2; pcoef(2) = Rg+Cp(1).Rj(1)+2*(Lg+Cp(1).Lj(1))/T+Cp(1).Rcm(1); Cp(1).Lg = Lg; Cp(1).Rg = Rg; Cp(1).Rg2 = Rg2; pcoef(3) = Ugim1;    
    try
        if const.osc_tg
            pcoef(3) = booltt; pcoef(4) = ptt; pcoef(5) = Rgtt2(1); pcoef(6) = utt; pcoef(7) = uttp1;  
            [ Aco, Qp, Ug(k), Uch(k) ] = esmfsolver_trill( pstmp, Cp, Qp, pcoef, T, Coup, bool ); 
        else
            [ Aco, Qp, Ug(k), Uch(k) ] = esmfsolver( pstmp, Cp, Qp, pcoef, T, Coup, bool );        
        end
    catch       
        break   
    end
     
    for kpipe = 1:Npipe    
        Acop(kpipe).P(:,k) = Aco(kpipe).P;
        Acop(kpipe).U(:,k) = Aco(kpipe).U;        
    end   

    Psup = Aco(1).P(1)+(Cp(1).Rj(1)+2/T*Cp(1).Lj(1)+Cp(1).Rcm(1))*(Ug(k)+Uch(k))-Qug; % supraglottal pressure
    Qug = 4/T*Cp(1).Lj(1)*(Ug(k)+Uch(k))-Qug;    
    if sgbool
        pstmp = Aco(isg).P(end-1);
    end    
    Um = sum(arrayfun(@(x) sum(x.U(end)), Aco(posrad))); % radiated volume velocity
    if const.selfoscil
        if strcmp(const.model,'smooth')
            [ Fvf, Ugim1 ] = force2mass( xvec, Hvec, pstmp, Psup, Ug(k), Ugim1, T, lg, const );
            [ yt, ytm1, ytm2 ] = displace2mass( Fvf, ytm1, ytm2, 1/T, mvfo, ke, re, kco );
        else
            if k <= 1; Qu1 = 0; Qu2 = 0; end            
            [ yt, ytm1, ytm2, Qu1, Qu2 ] = twomass_displace( ytm1, ytm2, Hvec, agovf(k), rgv, lgv, T, pstmp, Ug(k), mvfo, ke, kco, lg, Qu1, Qu2, rho, vfparam(1) );
        end
        
    end    
    Pm(k) = Um-Uk1;
    Uk1 = Um;
    
    if const.osc_tg
        %%%%%% Tongue tip
        utt = Acop(1).U(ptt,k); uttp1 = Acop(1).U(ptt+1,k); uttm1 = Acop(1).U(ptt-1,k);
        psub_tt = Acop(1).P(ptt-1,k)-(Cp(1).Rj(ptt-1)+2/T*Cp(1).Lj(ptt-1)+Cp(1).Rcp(ptt-1))*utt+Qtt;
        psup_tt = Acop(1).P(ptt+1,k)+(Cp(1).Rj(ptt+1)+2/T*Cp(1).Lj(ptt+1)+Cp(1).Rcm(ptt+1))*uttp1-Qttp1;
        Qtt = 4/T*Cp(1).Lj(ptt-1)*utt-Qtt;
        Qttp1 = 4/T*Cp(1).Lj(ptt+1)*uttp1-Qttp1;
        if const.nmass == 2
            if strcmp(const.model_tt,'smooth')
                [ Fvftt, Uttim1 ] = force2mass( xtvec, httvec, psub_tt, psup_tt, uttp1, Uttim1, T, dlt, const );
                [ ytt, yttm1, yttm2 ] = displace2mass( Fvftt, yttm1, yttm2, 1/T, mvfott, kett, rett, kcott );
            else
                if k <= 1; Qu1tt = 0; Qu2tt = 0; end
                rgvtt = [Cp(1).Rj(ptt-1),Cp(1).Rj(ptt)]; lgvtt = [Cp(1).Lj(ptt-1),Cp(1).Lj(ptt)];
                [ ytt, yttm1, yttm2, Qu1tt, Qu2tt ] = twomass_displace( yttm1, yttm2, httvec, agott(k), rgvtt, lgvtt, T, psub_tt, uttm1, mvfott, kett, kcott, dlt, Qu1tt, Qu2tt, rho, vfparam(2) );
            end
            ytt(1,:) = 0; yttm1(1,:) = 0; yttm2(1,:) = 0;
        else
            [ Fvftt, Uttim1 ] = force1mass( xtvec, httvec, psub_tt, psup_tt, uttp1, Uttim1, T, dlt, const );
            [ ytt, yttm1, yttm2 ] = displace1mass(  Fvftt, yttm1, yttm2, 1/T, mvfott(2,1), kett(2,1), rett(2,1) );
        end
    end
    Udcout(:,k) = [ Udc; Reyn; fpeak ]; 
    clear Cp 
end
Pout = Pm;
Ugout = Ug;
end

function [ Rg2, Rg, Lg, hsi, Udc ] = glotZupdate( xvec, Hvec, Amat, lmat, lg, Ps, const, Ach )
% Update the glottis impedance

selfmod = const.model;
rho = const.rho;
co_mu = const.mu;
sep = const.sep;
armin = const.amin;

%%% Glottis shape
x0 = xvec(1); x1 = xvec(2); x2 = xvec(3); dgl = xvec(end)-xvec(1);
h0 = Hvec(1); h1i = Hvec(2); h2i = Hvec(3);
[Ack, Ick ] = min(Amat);% Ick = min(length(Amat),Ick+14); %position
% Ick = Ick+20; 
Ac2 = 1/Ack.^2;
lc = lmat(Ick);
% Iup = find(Amat(Ick:end)<Ack+0.1*1e-4,1,'last'); if ~isempty(Amat(Ick:Ick+Iup-1)>=Ack+0.1*1e-4); Iup = 1; end ;Iup = Iup+Ick-1;
% Ilow = find(Amat(Ick:-1:1)>Ack+0.1*1e-4,1,'first'); if ~isempty(Amat(Ick:-1:Ick-Ilow+2)<=Ack+0.1*1e-4); Ilow = 2; end; Ilow = Ick-Ilow+2; 
% lc = sum(lmat(Ilow:Iup)); Ac2 = 1/(mean(Amat(Ilow:Iup))^2);

if strcmp(selfmod,'smooth')
    A(1) = (h1i-h0)/(x1-x0); % slope, part 1
    A(2) = (h2i-h1i)/(x2-x1); % slope, part 2
    
    %%%% Separation point    
    if (h1i>0) && (h2i>0) % No contact between masses
        if (h1i~=h2i) % if h1 different than h2
            hsi = sep*h1i;
        else
            hsi = h2i; % separation height is h2
        end        
        if (h1i>=h2i) || (hsi>=h2i) %
            hsi = h2i;
        end
    else
        hsi = armin/lg;   % if contact, separation height is 0 (set to eps to avoid singularities)
    end
    
    Ag22 = 1/(hsi*lg+Ach)^2;
    Rg2 = 0.5*rho/(lg^2)*(1/(hsi^2)-1/(h0^2));  % Bernoulli
    
    if (h1i~=h2i) % if plate 2 is non-horizontal
        Rg = 0.5*12*co_mu/lg*(1/A(1)*(1/(h0^2)-1/(h1i^2)) + 1/A(2)*(1/(h1i^2)-1/(hsi^2))); % viscous term
        Lg = rho/lg*(1/A(1)*log(h1i/h0)+1/A(2)*log(hsi/h1i)); % inertia
    else  % if plate 2 is horizontal
        Rg = 12*co_mu/lg*(0.5/A(1)*(1/(h0^2)-1/(h1i^2)) + (x2-x1)/(h1i^3)); % viscous term
        Lg = rho/lg*(1/A(1)*log(h1i/h0)+(x2-x1)/h1i); % inertia
    end
elseif strcmp(selfmod,'ishi')
    ag1 = h1i*lg; ag2 = h2i*lg;
    Rk1 = 0.19*rho/(ag1)^2;
    Rk2 = (0.5 - ag2/Amat(1)*(1-ag2/Amat(1))) / (ag2^2)*rho;
    Lg1 = rho*(x1-x0)/ag1;
    Lg2 = rho*(x2-x1)/ag2;
    Rv1 = 12*co_mu*lg^2*(x1-x0)/(ag1^3);
    Rv2 = 12*co_mu*lg^2*(x2-x1)/(ag2^3);
    Rg2 = Rk1+Rk2;
    Rg = [Rv1, Rv2];
    Lg = [Lg1,Lg2];
    hsi = min(h1i,h2i); 
    hsi = max(hsi,armin/lg);
%     if (h1i>0) && (h2i>0) % No contact between masses
%         if (h1i~=h2i) % if h1 different than h2        
%             hsi = min(sep*h1i,h2i); % separation height is h2
%         end        
%         if (h1i>=h2i) || (hsi>=h2i) %
%             hsi = h2i;
%         end
%     else
%         hsi = armin/lg;   % if contact, separation height is 0 (set to eps to avoid singularities)
%     end    
    Ag22 = 1/(hsi*lg+Ach)^2;
end

rv_sum = (12*lg*dgl*Ag22.*sqrt(Ag22)+8*pi*lc.*Ac2)*co_mu;
% rv_sum = 12*(dgl*Ag22+lc.*Ac2)*co_mu;
rk_sum = 1.38*rho*(Ac2+Ag22);
% rv_sum = Rg+8*pi*lc.*Ac2*co_mu;
% rk_sum = 1.38*rho*Ac2+Rg2;
Udc = (-rv_sum+sqrt(rv_sum*rv_sum+4*rk_sum*Ps))/(2*rk_sum); 	
if ( isempty(Udc) || isnan(Udc) ); Udc = 0; end

end

function [ Rg2, Rg, Lg, hsi ] = tonguetipZupdate( xvec, Hvec, lg, const )
% Update the glottis impedance

selfmod = const.model_tt;
rho = const.rho;
co_mu = const.mu;
sep = const.sep;
armin = const.amin;

%%% Glottis shape
x0 = xvec(1); x1 = xvec(2); x2 = xvec(3); %dgl = xvec(end)-xvec(1); xm = (x2+x1)/2;
h0 = Hvec(1); h1i = Hvec(2); h2i = Hvec(3); h3i = Hvec(4);
ap1 = h3i*lg;

if strcmp(selfmod,'smooth')
    A(1) = (h1i-h0)/(x1-x0); % slope, part 1
    A(2) = (h2i-h1i)/(x2-x1); % slope, part 2
    
    %%%% Separation point
    
    if (h1i>0) && (h2i>0) % No contact between masses
        if (h1i~=h2i) % if h1 different than h2
            hsi = sep*h1i;
        else
            hsi = h2i; % separation height is h2
        end        
        if (h1i>=h2i) || (hsi>=h2i) %
            hsi = h2i;
        end
    else
        hsi = armin/lg;   % if contact, separation height is 0 (set to eps to avoid singularities)
    end    
    Rg2 = 0.5*rho/(lg^2)*(1/(hsi^2)-1/(h0^2));  % Bernoulli
    
    if (h1i~=h2i) % if plate 2 is non-horizontal
        Rg = 0.5*12*co_mu/lg*(1/A(1)*(1/(h0^2)-1/(h1i^2)) + 1/A(2)*(1/(h1i^2)-1/(hsi^2))); % viscous term
        Lg = rho/lg*(1/A(1)*log(h1i/h0)+1/A(2)*log(hsi/h1i)); % inertia
    else  % if plate 2 is horizontal
        Rg = 12*co_mu/lg*(0.5/A(1)*(1/(h0^2)-1/(h1i^2)) + (x2-x1)/(h1i^3)); % viscous term
        Lg = rho/lg*(1/A(1)*log(h1i/h0)+(x2-x1)/h1i); % inertia
    end
elseif strcmp(selfmod,'ishi')
    ag1 = h1i*lg; ag2 = h2i*lg;
    Rk1 = 0.19*rho/(ag1)^2;
    Rk2 = (0.5 - ag2/ap1*(1-ag2/ap1)) / (ag2^2)*rho;
    Lg1 = rho*(x1-x0)/ag1;
    Lg2 = rho*(x2-x1)/ag2;
    Rv1 = 12*co_mu*lg^2*(x1-x0)/(ag1^3);
    Rv2 = 12*co_mu*lg^2*(x2-x1)/(ag2^3);
    Rg2 = Rk1+Rk2;
    Rg = [Rv1, Rv2];
    Lg = [Lg1,Lg2];
    hsi = min(h1i,h2i); 
    if (h1i>0) && (h2i>0) % No contact between masses
        if (h1i~=h2i) % if h1 different than h2        
            hsi = min(sep*h1i,h2i); % separation height is h2
        end        
        if (h1i>=h2i) || (hsi>=h2i) %
            hsi = h2i;
        end
    else
        hsi = armin/lg;   % if contact, separation height is 0 (set to eps to avoid singularities)
    end   
end

end

function [ mvfo, kvfo, rvfo, kco ] = VFparamupdate(fo, param, alphaglot)
% Adapt the values of the mass and stiffness of vocal folds to match the
% fundamental frequency fo

mvfi = param.Mi;
Qr = param.Qr;
Qrc = param.Qrc;
if nargin < 6; alphaglot = 1; end
if (isinf(fo) || isnan(fo) || fo <= 1 ); fo = 1 ; end
mvfo = mvfi*alphaglot;
kvfo = 4*pi^2*fo^2*mvfo; 
kco = [ kvfo(1,1); kvfo(2,1) ]*Qrc;
rvfo = Qr*sqrt(kvfo.*mvfo/2);

end

function [ ke, re ] = contactparam(mvf, kvf, rvf, Hvec, const, org)
% Compute new stiffness and damping if contact
%
% Input arguments
%
% mvf : 2x2 matrix containing the mass values
% kvf : 2x2 matrix containing the stifness values
% rvf : 2x2 matrix containing the damping values
% Hvec : vector containing the height of the masses
%
% Output arguments
%
% ke : new stiffness
% re : new damping

blim = const.blim;

if nargin < 6; org = 'vf'; end

% Inialization of output arguments
ke = kvf; re = rvf;
if strcmp(org,'vf')
    kctc = const.kctc;
    rctc = const.rctc;
else
    kctc = const.kctctt;
    rctc = const.rctctt;
end

if (Hvec(2) <= blim)   % contact at the 1st mass location    
    ke(:,1) = kctc*kvf(:,1); % New stiffness for 1st mass
    re(:,1) = rctc*sqrt(kvf(:,1).*mvf(:,1)/2);%rvf(:,1)+2*sqrt(kvf(:,1).*mvf(:,1)); % New damping for 1st mass    
end

if (Hvec(3) <= blim) % contact at the 2nd mass location    
    ke(:,2) = kctc*kvf(:,2); % New stiffness for 2nd mass
    re(:,2) = rctc*sqrt(kvf(:,2).*mvf(:,2)/2);%rvf(:,2)+2*sqrt(kvf(:,2).*mvf(:,2)); % New damping for 2nd mass
end
 
end

function [ Fvf, Ugim1 ] = force2mass( xvec, Hvec, Ps, Psup, Ug, Ugim1, T, lg, const)
% Compute the applied force on vocal folds

sep = const.sep;

%%% Glottis shape
x0 = xvec(1); x1 = xvec(2); x2 = xvec(3); x3 = xvec(4);
h0 = Hvec(1); h1i = Hvec(2); h2i = Hvec(3); h3 = Hvec(4);

DeltaPi = Ps-Psup;

A(1) = (h1i-h0)/(x1-x0); % slope, part 1
A(2) = (h2i-h1i)/(x2-x1); % slope, part 2
A(3) = (h3-h2i)/(x3-x2); % slope, part 3
B(1) = h0-A(1)*x0;  % y-intercept, part 1
B(2) = h1i-A(2)*x1; % y-intercept, part 2
B(3) = h2i-A(3)*x2; % y-intercept, part 3

%%%% Separation point

if (h1i>0) && (h2i>0) % No contact between masses
    if (h1i~=h2i) % if h1 different than h2
         hsi = sep*h1i; 
         xsi = (hsi-h1i)/A(2)+x1;
    else
         xsi = x2; % if h1 = h2
         hsi = h2i; % separation height is h2
    end
    if (h1i>=h2i) || (hsi>=h2i) %
        hsi = h2i;
        xsi = x2;   % separation point is x2
    end
else    
    hsi = eps;   % if contact, separation is 0
    xsi = eps;   %    
end

if DeltaPi>=0
    dUg_tmp = (Ug-Ugim1)/T;
    Fvf = forcecompute(Ug,lg,dUg_tmp,Ps,Psup,xvec,h1i,h2i,A,B,h0,xsi,hsi, const);
else
    dUg_tmp = (-Ug-Ugim1)/T;
    Fvf = forcecompute(Ug,lg,dUg_tmp,Psup-Ps,0,xvec,h2i,h1i,A,B,h0,xsi,hsi, const);
end

Ugim1 = Ug;

end



function Reyn = Reynold_calc( a, Udc, rho, co_mu, gcrit )
% Compute the Reynolds number inside the constriction of the waveguide
% defined by a, l.
%
% Input arguments :
%
% a : area function (in  m^2)
% Udc : volume velocity
% rho : air volume density (in kg/m^3, default = 1.024)
% co_mu : air viscosity coefficient (default = 1.85e-5)
% gcrit : lower bound for the constriction position, default = 20
%
% Output argument :
%
% Reyn : Reynolds number
%
% Benjamin Elie, january 2013
%

if nargin < 3; rho = 1.024; end
if nargin < 4; co_mu = 1.85e-5; end
if nargin < 5; gcrit = 1; end

% find the constriction geometry
Ack = min(a(gcrit:end)); 
Reyn = 2*rho/co_mu*Udc/sqrt(pi*Ack);

end

function [ Vc, Q, V, Qwl, Qwc, Qnc ] = Init_Qp(Nx)
%%%%% Initialisation of the derivative and integration terms
% Nx is the number of tubelets of the considered pipe

Vc = zeros(Nx,1);
Q = zeros(Nx+1,1);
V = zeros(Nx+1,1);
Qwl = Vc;
Qwc = Vc;
Qnc = 0;

end

function [ Lj, Rj, Cj, Srad, Gw, wl, wc, Udj, bj, Hj, Rcm, Rcp ] = acou2elec( Xj, Aj, const, ujm1, aglot, subgl )
% Computes the electric components values of the vocal tract

if nargin < 6; subgl = 0; end

[ M, Nt ] = size(Aj);
A_lip = Aj(end,:);
rho = const.rho;
co_mu = const.mu;
c = const.c;
T = const.T;
rmlt = const.rmlt;
wall_resi = const.wr;
wall_mass = const.wm;
wall_comp = const.wc;
rad = sqrt(Aj/pi);
aprec = diff([aglot;Aj(:)]);
asucc = sign(diff([Aj(:);1e10]));

Gw = zeros(size(Xj));
Udj = zeros(size(Xj));
Rcm = Udj;
Rcp = Udj;
if const.dynterm
    Rcm(aprec<0) = abs(ujm1(aprec<0))*const.rho./(2*Aj(aprec<0).^2);
    Rcp(asucc<0) = -abs(ujm1(find(asucc<0)+1))*const.rho./(2*Aj(asucc<0).^2);
end

Lj = rho*Xj/2./Aj;
Rj = rmlt*pi*co_mu.*Xj./Aj.^2;
Cj = Xj.*Aj/(rho*c^2);
Srad = T/2*const.S*pi*sqrt(pi)/rho;
Grad = const.G*pi^2/rho/c;
if ~subgl
    wr = wall_resi/(2*sqrt(pi))./Xj./sqrt(Aj);
    wl = 2/T*wall_mass/(2*sqrt(pi))./Xj./sqrt(Aj);
    wc = T/2./(wall_comp/(2*sqrt(pi))./Xj./sqrt(Aj));
else
    ltrach = sum(Aj);
    xtrach = cumsum(Aj)/ltrach;
    wr = 0.33./(1.30.^(xtrach/4))*1.6e3./(2*pi*(rad.^3).*Xj);
    wl = 0.33./(1.30.^(xtrach/4))*1.1e-3./(2*pi*rad.*Xj);
    wc = (2*pi*(rad.^3).*Xj)./(0.33./(1.30.^(xtrach/4))*0.392e6);
end

if const.wallyield == 1; Gw = 1./(wr+wl+wc); end

%%%% Compute linear system coefficients
bj = zeros(M+1, Nt);
Hj = zeros(M+1, Nt);
bj(1:M,:) = 1./(2*Cj/T+Gw);
Hj(2:M,:) = -2*(Lj(1:M-1,:)+Lj(2:M,:))/T-Rj(1:M-1,:)-Rj(2:M,:)-Rcm(1:M-1,:)-Rcp(2:M,:)-bj(1:M-1,:)-bj(2:M,:);
bj(M+1,:) = 1./(Srad*sqrt(A_lip)+Grad*A_lip);
Hj(M+1,:) = -(2/T*Lj(M,:)+Rj(M,:)+bj(M,:)+bj(M+1,:)+Rcp(M));

end

function [ yt, ytm1, ytm2, Qu1, Qu2 ] = twomass_displace( ytm1, ytm2, hvec, ago, Rg, Lg, T, Ps, Ug, mvfo, ke, kco, lg, Qu1, Qu2, rho, param )

%%% Glottis shape

% %%% Is it tongue of glottis ?
% if isfield(param,'pos')
%     if ~isempty(param.pos)
%         vf = 0;
%     else
%         vf = 1;
%     end
% else
%     vf = 1;
% end

xvec = param.xi;
x0 = xvec(1); x1 = xvec(2); x2 = xvec(3); x3 = xvec(4);
d1 = x1-x0; d2 = x2-x1;
h0 = hvec(1); h1i = hvec(2); h2i = hvec(3); h3 = hvec(4);
% ag1m1 = h1i*lg;
% ag2m1 = h2i*lg;
% if ~vf; disp(ag1m1); disp(ag2m1); end
ag1m1 = sum(ytm1(:,1))*lg+ago;
ag2m1 = sum(ytm1(:,2))*lg+ago;
ago1 = ago;% sum(vfparam(1).y0(:,1))*lg;
ago2 = ago;% sum(vfparam(1).y0(:,2))*lg;
etha_k1 = param.etha_k1;
etha_k2 = param.etha_k2;
etha_h1 = param.etha_h1;
etha_h2 = param.etha_h2;
U1 = Ug;
m1 = mvfo(2,1); m2 = mvfo(2,2);
k1 = ke(2,1); k2 = ke(2,2);
h1 = 3*k1; h2 = 3*k2;
kc = kco(1);
y1m1 = sum(ytm1(:,1)); y2m1 = sum(ytm1(:,2)); 
y1m2 = sum(ytm2(:,1)); y2m2 = sum(ytm2(:,2)); 

% Pressure acting on m1
Pm1 =  Ps - 1.37*rho/2*(U1/(ag1m1))^2 - ...
1/2*( Rg(1)*U1+2*Lg(1)/T*U1-Qu1 );
Qu1 = 4/T*Lg(1)*U1-Qu1;
F1 = d1*lg*Pm1; % Force due to Pm1

% Pressure acting on m2
F2 = d2*lg*( Pm1 - 1/2*(sum(Rg))*U1-2*(sum(Lg))/T*U1+Qu2...
-rho/2*U1^2*( 1/ag2m1^2-1/ag1m1^2 ) );
Qu2 = 4/T*(sum(Lg))*U1-Qu2;

% Damping ratio
r1 = 2*0.1*sqrt(m1.*k1);
r2 = 2*0.6*sqrt(m2.*k2);

% if k ==  2
% 
% y1(k) = F1 / ( m1/T^2+r1/T+k1 );
% x2(k) = F2 / ( m2/T^2+r2/T+k2 );
% 
% else

% Computation of every possible case.
%1 and 2 open
y1oo = ( m1/T^2*(2*y1m1-y1m2) + ...
    r1/T*y1m1-k1*etha_k1*y1m1^3 -...
    kc*(y1m1-y2m1) + F1)...
    /( m1/T^2+r1/T+k1 );
y2oo = ( m2/T^2*(2*y2m1-y2m2) + ...
    r2/T*y2m1-k2*etha_k2*y2m1^3 -...
    kc*(y2m1-y1m1) + F2)...
    /( m2/T^2+r2/T+k2 );

% 1 open and 2 closed
r2 = 2*1.6*sqrt(m2*k2);
y1of = ( m1/T^2*(2*y1m1-y1m2) + ...
    r1/T*y1m1-k1*etha_k1*y1m1^3 -...
    kc*(y1m1-y2m1) +Ps*d1*lg)...
    /( m1/T^2+r1/T+k1 );
y2of = ( m2/T^2*(2*y2m1-y2m2) + ...
    r2/T*y2m1-k2*etha_k2*y2m1^3 -...
    kc*(y2m1-y1m1) + Ps*d2*lg - ...
    h2*(ago2/2/lg+etha_h2*(y2m1+ago2/2/lg)^3) )...
    /( m2/T^2+r2/T+k2+h2);

% 1 closed and 2 open
r1 = 2*1.1*sqrt(m1*k1);
r2 = 2*0.6*sqrt(m2*k2);
y1fo = ( m1/T^2*(2*y1m1-y1m2) + ...
    r1/T*y1m1-k1*etha_k1*y1m1^3 -...
    kc*(y1m1-y2m1) + Ps*d1*lg - ...
    h1*(ago1/2/lg+etha_h1*(y1m1+ago1/2/lg)^3))...
    /( m1/T^2+r1/T+k1+h1 );
y2fo = ( m2/T^2*(2*y2m1-y2m2) + ...
    r2/T*y2m1-k2*etha_k2*y2m1^3 -...
    kc*(y2m1-y1m1) + 0)...
    /( m2/T^2+r2/T+k2 );

% 1 and 2 closed
r2 = 2*1.6*sqrt(m2*k2);
y1ff = ( m1/T^2*(2*y1m1-y1m2) + ...
    r1/T*y1m1-k1*etha_k1*y1m1^3 -...
    kc*(y1m1-y2m1) + Ps*d1*lg - ...
    h1*(ago1/2/lg+etha_h1*(y1m1+ago1/2/lg)^3))...
    /( m1/T^2+r1/T+k1+h1 );
y2ff = ( m2/T^2*(2*y2m1-y2m2) + ...
    r2/T*y2m1-k2*etha_k2*y2m1^3 -...
    kc*(y2m1-y1m1) + 0 - ...
    h2*(ago2/2/lg+etha_h2*(y2m1+ago2/2/lg)^3) )...
    /( m2/T^2+r2/T+k2+h2);

if y1oo>-ago1/(2*lg)
    if y2oo>-ago2/(2*lg)
        y1 = y1oo;
        y2 = y2oo;
    else
        y1 = y1of;
        y2 = y2of;
    end
else
    if y2fo>-ago2/(2*lg)
        %           1 closed 2 open
        y1 = y1fo;
        y2 = y2fo;
    else
        %           1 closed 2 closed
        y1 = y1ff;
        y2 = y2ff;
    end
end

ytm2 = ytm1;
yt = [y1/2*ones(2,1), y2/2*ones(2,1)];
ytm1 = yt;
    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%% The following functions are from N. Ruty %%%%%%%%%%%%%%%%%
%%% "N. Ruty. Modèles d'interactions fluide/parois dans le conduit vocal. 
%%% Applications aux voix et aux pathologies. PhD thesis, 2007 %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Fvf = forcecompute(Ug,lg,d_Ug_dt,Ps,Psupra,xvec,h1,h2,A,B,h0,xs,hs,const)
% calcul des 4 forces at the points i=0,1,2,3
% l: reaction forces resulting from the pressure distribution in the flow channel LEFT from the point i
% r: reaction forces resulting from the pressure distribution in the flow channel RIGHT from the point i
% formules Annexe C Vilain, 


% modification par NR le 02/04/07
%%%%%%%%%%%%% New modifications by B. Elie, 2016
%
% d�veloppement limit� de Fr_h1 et Fl_h2, lorsque le rapport A(2)*x2/B(2) tend vers 0 (<pres ,
% pres fix� dans init_gen) , ligne 78-89, 116-128, 193-206
%
% Fonction pour le calcul de la contribution des forces de pression sur les
% masses 1 et 2
% 
% Param�tres d'entr�e:
% Ug: D�bit glottique
% Lg: largeur de la glotte,
% d_Ug_dt: derivee du d�bit glottique
% P: pression sous-glottique
% Psupra: pression supra glottique
% xvec: vecteur contenant les abscisses des masses
% h1,h2: ouvertures au niveau des masses 1 et 2
% A,B: coeficients des �quations des plaques
% h0: ouverture en entr�e/sortie de constriction
% Visco,Instat: indicateurs de prise en compte ou non de la viscosit� et de l'inertie
% xs: abscisse du point de s�paration de l'�coulement
% hs: ouverture au niveau du point de s�paration
% 
% Param�tres de sortie:
% 
% Fl_h2: force de pression sur la masse 2 � gauche
% Fr_h2: force de pression sur la masse 2 � droite
% Fl_h1: force de pression sur la masse 1 � gauche
% Fr_h1: force de pression sur la masse 1 � droite

rho = const.rho;
co_mu = const.mu;
sep = const.sep;
% initialisation des forces
Fl_h2 = 0; Fr_h1 = 0;

% assignation des abscisses � partir du vecteur xvec
x0 = xvec(1); x1 = xvec(2); x2 = xvec(3); x3 = xvec(4);
pres = 1e-5;
% quelques variables temporaires
tmp1  = 0.5*rho*(Ug^2)/(lg^2); %variable temporaire associ�e au terme de Bernoulli
tmp2  = -12*co_mu*Ug/lg; %variable temporaire associ�e au terme de Poiseuille
tmp3  = -rho*d_Ug_dt/lg; %variable temporaire li�e au terme d'inertie

Fl_h1 = 0.5*(x1-x0)*lg*Ps; %force sur la masse 1 � gauche, temporaire, valable en cas de fermeture

if (h1>0) && (h2>0) 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%% Fl_h1 %%%%%%%%%%%%%%%%%%%%%
    i1 = 1;
    bern = tmp1*(1/(h0^2)-2*(Xv1(x0,x1,A,B,i1)-x0*Wv1(x0,x1,A,B,i1))/(x1-x0)^2); %     Bernoulli 
    pois = tmp2/(2*A(1))*(1/(h0^2)-2*(Xv1(x0,x1,A,B,i1)-x0*Wv1(x0,x1,A,B,i1))/(x1-x0)^2);%     Poiseuille
    inst = tmp3/A(1)*(2*(Zv1(x0,x1,A,B,i1)-x0*Yv1(x0,x1,A,B,i1))/(x1-x0)^2-log(h0)); %     Inertia   
    Fl_h1 = 0.5*(x1-x0)*lg*(Ps+bern+pois+inst);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%% Fr_h1 %%%%%%%%%%%%%%%%%%%%%
    %%% two cases :::: h2 < h1*sep and h2 > h1*sep %%%
    if (h2 <h1*sep)
        if (abs(A(2)*x2/B(2))<pres) % Parrallel duct
            i2 = 2;
%             bern  = tmp1*(1/(h0^2)-1/(h1^2));
            bern = tmp1*(1/(h0^2)+2*(Xv1(x1,x2,A,B,i2)-x2*Wv1(x1,x2,A,B,i2))/(x2-x1)^2); %  Bernoulli
%             pois = tmp2*(1/(2*A(1))*(1/(h0^2)-1/(h1^2))+(x2-x1)/(h1^3));
            pois = tmp2*(1/(2*A(1))*(1/(h0^2)-1/(h1^2))-1/B(2)^3*(x1-3/2*A(2)/B(2)*x1^2+2*A(2)^2/B(2)^2*x1^3)...
                -2/(x2-x1)^2/B(2)^3*((x2^3/3-3/8*A(2)/B(2)*x2^4+2/5*A(2)^2/B(2)^2*x2^5-...
                                       x2*(x2^2/2-1/2*A(2)/B(2)*x2^3+1/2*A(2)^2/B(2)^2*x2^4))...
                                       -(x1^3/3-3/8*A(2)/B(2)*x1^4+2/5*A(2)^2/B(2)^2*x1^5-...
                                       x2*(x1^2/2-1/2*A(2)/B(2)*x1^3+1/2*A(2)^2/B(2)^2*x1^4)))); % Poiseuille
%             inst = tmp3*(log(h1ia/h0)/A(1)+(x2-x1)/h1);
            inst = tmp3*(log(h1/h0)/A(1)-1/B(2)*(x1-1/2*A(2)/B(2)*x1^2+1/3*A(2)^2/B(2)^2*x1^3)-...
                2/(x2-x1)^2/B(2)*((1/3*x2^3-1/8*A(2)/B(2)*x2^4+1/15*A(2)^2/B(2)^2*x2^5-...
                                  x2*(1/2*x2^2-1/6*A(2)/B(2)*x2^3+1/12*A(2)^2/B(2)^2*x2^4))...
                                  -(1/3*x1^3-1/8*A(2)/B(2)*x1^4+1/15*A(2)^2/B(2)^2*x1^5-...
                                  x2*(1/2*x1^2-1/6*A(2)/B(2)*x1^3+1/12*A(2)^2/B(2)^2*x1^4)))); % Inertia
              %proposition modif par NR 11/04/2007
%             inst = tmp3*(log(h1/h0)/A(1)+(x2-x1)/3/h1);            
        else %glotte formant un canal convergent ou divergent avec h2<hs
            i2 = 2; %num�ro de la plaque sur laquelle on calcule les forces
            bern = tmp1*(1/(h0^2)+2*(Xv1(x1,x2,A,B,i2)-x2*Wv1(x1,x2,A,B,i2))/(x2-x1)^2);% Bernoulli
            pois = tmp2*(1/(2*A(1))*(1/(h0^2)-1/(h1^2))+1/(2*A(2))*(1/(h1^2)+2*(Xv1(x1,x2,A,B,i2)-x2*Wv1(x1,x2,A,B,i2))/(x2-x1)^2));% Poiseuille
            inst = tmp3*(log(h1/h0)/A(1) + 1/A(2)*(2*(x2*Yv1(x1,x2,A,B,i2)-Zv1(x1,x2,A,B,i2))/(x2-x1)^2-log(h1)));% Inertia
        end
        
        Fr_h1 = 0.5*(x2-x1)*lg*(Ps+bern+pois+inst);
    else   %cas o� la glotte forme un canal divergent avec h2>hs
        if (abs(A(2)*x2/B(2))<pres)
        i2 = 2; %num�ro de la plaque sur laquelle on calcule les forces
        %hs   = sep*h1;
        %xs   = (hs-h1)/A(2)+x1;
        tmpG = (xs-x1)/(x2-x1)*(x2-(xs+x1)/2); %variable temporaire
%         tmpG = 0.5*((x2-x1)+(xs-x2)^2/(x1-x2));   %modif par NR 02/04/2007
       %             bern  = tmp1*(1/(h0^2)-1/(h1^2));
            bern = tmp1*(1/(h0^2)-(x2*Wv1(x1,xs,A,B,i2)-Xv1(x1,xs,A,B,i2))/(tmpG*(x2-x1)));
%             terme de poiseuille
%             pois = tmp2*(1/(2*A(1))*(1/(h0^2)-1/(h1^2))+(x2-x1)/(h1^3));
            pois = tmp2*(1/(2*A(1))*(1/(h0^2)-1/(h1^2))-1/B(2)^3*(x1-3/2*A(2)/B(2)*x1^2+2*A(2)^2/B(2)^2*x1^3)...
                -2/(x2-x1)/tmpG/B(2)^3*((xs^3/3-3/8*A(2)/B(2)*xs^4+2/5*A(2)^2/B(2)^2*xs^5-...
                                       x2*(xs^2/2-1/2*A(2)/B(2)*xs^3+1/2*A(2)^2/B(2)^2*xs^4))...
                                       -(x1^3/3-3/8*A(2)/B(2)*x1^4+2/5*A(2)^2/B(2)^2*x1^5-...
                                       x2*(x1^2/2-1/2*A(2)/B(2)*x1^3+1/2*A(2)^2/B(2)^2*x1^4)))); 

%             terme d'inertie
%             inst = tmp3*(log(h1/h0)/A(1)+(x2-x1)/h1);
            inst = tmp3*(log(h1/h0)/A(1)-1/B(2)*(x1-1/2*A(2)/B(2)*x1^2+1/3*A(2)^2/B(2)^2*x1^3)...
                -2/(x2-x1)/tmpG/B(2)*((1/3*xs^3-1/8*A(2)/B(2)*xs^4+1/15*A(2)^2/B(2)^2*xs^5-...
                                  x2*(1/2*xs^2-1/6*A(2)/B(2)*xs^3+1/12*A(2)^2/B(2)^2*xs^4))...
                                  -(1/3*x1^3-1/8*A(2)/B(2)*x1^4+1/15*A(2)^2/B(2)^2*x1^5-...
                                  x2*(1/2*x1^2-1/6*A(2)/B(2)*x1^3+1/12*A(2)^2/B(2)^2*x1^4))));
            
        else
        i2 = 2; %num�ro de la plaque sur laquelle on calcule les forces
        %hs   = sep*h1;
        %xs   = (hs-h1)/A(2)+x1;
        tmpG = (xs-x1)/(x2-x1)*(x2-(xs+x1)/2); %variable temporaire
%         tmpG = 0.5*((x2-x1)+(xs-x2)^2/(x1-x2));   %modif par NR 02/04/2007

%         terme de bernoulli
        bern = tmp1*(1/(h0^2)-(x2*Wv1(x1,xs,A,B,i2)-Xv1(x1,xs,A,B,i2))/(tmpG*(x2-x1)));
%         terme de poiseuille
        pois = tmp2*(1/(2*A(1))*(1/(h0^2)-1/(h1^2))+1/(2*A(2))*(1/(h1^2))-(x2*Wv1(x1,xs,A,B,i2)-Xv1(x1,xs,A,B,i2))/(tmpG*(x2-x1)));
%         terme d'inertie
        inst = tmp3*(log(h1/h0)/A(1)+1/A(2)*((x2*Yv1(x1,xs,A,B,i2)-Zv1(x1,xs,A,B,i2))/(tmpG*(x2-x1))-log(h1))); 
        end
        
        Fr_h1 = lg*tmpG*(Ps+bern+pois+inst)+lg*Psupra*(x2-xs)/(x2-x1)*(x2-(x2+xs)/2);
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%% Calcul de Fl_h2 %%%%%%%%%%%%%%%%%%%%%
    %%% two cases :::: h2 < h1*sep et le cas h2 > h1*sep %%%
    
    if (h2 < h1*sep)
        if (abs(A(2)*x2/B(2))<pres) %glotte formant un conduit parall�le
	i2 = 2;
%         terme de bernoulli
%             bern  = tmp1*(1/(h0^2)-1/(h1^2));
            bern = tmp1*(1/(h0^2)-2*(Xv1(x1,x2,A,B,i2)-x1*Wv1(x1,x2,A,B,i2))/(x2-x1)^2);
%         terme de poiseuille
%             pois = tmp2*(1/(2*A(1))*(1/(h0^2)-1/(h1^2))+(x2-x1)/(h1^3));
            pois = tmp2*(1/(2*A(1))*(1/(h0^2)-1/(h1^2))-1/B(2)^3*(x1-3/2*A(2)/B(2)*x1^2+2*A(2)^2/B(2)^2*x1^3)...
                +2/(x2-x1)^2/B(2)^3*((x2^3/3-3/8*A(2)/B(2)*x2^4+2/5*A(2)^2/B(2)^2*x2^5-...
                                       x1*(x2^2/2-1/2*A(2)/B(2)*x2^3+1/2*A(2)^2/B(2)^2*x2^4))...
                                       -(x1^3/3-3/8*A(2)/B(2)*x1^4+2/5*A(2)^2/B(2)^2*x1^5-...
                                       x1*(x1^2/2-1/2*A(2)/B(2)*x1^3+1/2*A(2)^2/B(2)^2*x1^4)))); 

%         terme d'inertie
%             inst = tmp3*(log(h1/h0)/A(1)+(x2-x1)/h1);
            inst = tmp3*(log(h1/h0)/A(1)-1/B(2)*(x1-1/2*A(2)/B(2)*x1^2+1/3*A(2)^2/B(2)^2*x1^3)+...
                2/(x2-x1)^2/B(2)*((1/3*x2^3-1/8*A(2)/B(2)*x2^4+1/15*A(2)^2/B(2)^2*x2^5-...
                                  x1*(1/2*x2^2-1/6*A(2)/B(2)*x2^3+1/12*A(2)^2/B(2)^2*x2^4))...
                                  -(1/3*x1^3-1/8*A(2)/B(2)*x1^4+1/15*A(2)^2/B(2)^2*x1^5-...
                                  x1*(1/2*x1^2-1/6*A(2)/B(2)*x1^3+1/12*A(2)^2/B(2)^2*x1^4))));
            
        else %glotte formant un canal convergent ou divergent avec h2<hs
            i2 = 2; %num�ro de la plaque sur laquelle on fait le calcul...
%         terme de bernoulli
            bern = tmp1*(1/(h0^2)-2*(Xv1(x1,x2,A,B,i2)-x1*Wv1(x1,x2,A,B,i2))/(x2-x1)^2);
%         terme de poiseuille	        
            pois = tmp2*(1/(2*A(1))*(1/(h0^2)-1/(h1^2))+1/(2*A(2))*(1/(h1^2)-2*(Xv1(x1,x2,A,B,i2)-x1*Wv1(x1,x2,A,B,i2))/(x2-x1)^2));
%         terme d'inertie	        
            inst = tmp3*(log(h1/h0)/A(1) +1/A(2)*(2*(Zv1(x1,x2,A,B,i2)-x1*Yv1(x1,x2,A,B,i2))/(x2-x1)^2-log(h1)));
        end
        
        Fl_h2 = 0.5*(x2-x1)*lg*(Ps+bern+pois+inst);
        
    else %cas o� la glotte forme un canal divergent avec h2>hs
        if (abs(A(2)*x2/B(2))<pres) %glotte formant un conduit parall�le
            i2 = 2;
            tmpH = (xs-x1)/(x2-x1)*((xs+x1)/2-x1); %variable temporaire
        %         terme de bernoulli
%             bern  = tmp1*(1/(h0^2)-1/(h1^2));
            bern = tmp1*(1/(h0^2)-(Xv1(x1,xs,A,B,i2)-x1*Wv1(x1,xs,A,B,i2))/(tmpH*(x2-x1)));
%         terme de poiseuille
%             pois = tmp2*(1/(2*A(1))*(1/(h0^2)-1/(h1^2))+(x2-x1)/(h1^3));
            pois = tmp2*(1/(2*A(1))*(1/(h0^2)-1/(h1^2))-1/B(2)^3*(x1-3/2*A(2)/B(2)*x1^2+2*A(2)^2/B(2)^2*x1^3)...
                +2/(x2-x1)/tmpG/B(2)^3*((xs^3/3-3/8*A(2)/B(2)*xs^4+2/5*A(2)^2/B(2)^2*xs^5-...
                                       x1*(xs^2/2-1/2*A(2)/B(2)*xs^3+1/2*A(2)^2/B(2)^2*xs^4))...
                                       -(x1^3/3-3/8*A(2)/B(2)*x1^4+2/5*A(2)^2/B(2)^2*x1^5-...
                                       x1*(x1^2/2-1/2*A(2)/B(2)*x1^3+1/2*A(2)^2/B(2)^2*x1^4)))); 

%         terme d'inertie
%             inst = tmp3*(log(h1/h0)/A(1)+(x2-x1)/h1);
            inst = tmp3*(log(h1/h0)/A(1)-1/B(2)*(x1-1/2*A(2)/B(2)*x1^2+1/3*A(2)^2/B(2)^2*x1^3)...
                +2/(x2-x1)/tmpH/B(2)*((1/3*xs^3-1/8*A(2)/B(2)*xs^4+1/15*A(2)^2/B(2)^2*xs^5-...
                                  x1*(1/2*xs^2-1/6*A(2)/B(2)*xs^3+1/12*A(2)^2/B(2)^2*xs^4))...
                                  -(1/3*x1^3-1/8*A(2)/B(2)*x1^4+1/15*A(2)^2/B(2)^2*x1^5-...
                                  x1*(1/2*x1^2-1/6*A(2)/B(2)*x1^3+1/12*A(2)^2/B(2)^2*x1^4))));
            
        else
        i2 = 2; %num�ro de la plaque sur laquelle on fait le calcul...
        %hs   = sep*h1;
        %xs   = (hs-h1)/A(2)+x1;
        tmpH = (xs-x1)/(x2-x1)*((xs+x1)/2-x1); %variable temporaire
%         terme de bernoulli
        bern = tmp1*(1/(h0^2)-(Xv1(x1,xs,A,B,i2)-x1*Wv1(x1,xs,A,B,i2))/(tmpH*(x2-x1)));
%         terme de poiseuille
        pois = tmp2*(1/(2*A(1))*(1/(h0^2)-1/(h1^2))+1/(2*A(2))*(1/(h1^2))-(Xv1(x1,xs,A,B,i2)-x1*Wv1(x1,xs,A,B,i2))/(tmpH*(x2-x1)));
        %proposition NR, 11/04/2007;
%         pois = tmp2*(1/(2*A(1))*(1/(h0^2)-1/(h1^2))+1/(2*A(2))*(1/(h1^2)-(Xv1(x1,xs,A,B,i2)-x1*Wv1(x1,xs,A,B,i2))/(tmpH*(x2-x1))));
%         terme d'inertie	
        inst = tmp3*(log(h1/h0)/A(1) +1/A(2)*((Zv1(x1,xs,A,B,i2)-x1*Yv1(x1,xs,A,B,i2))/(tmpH*(x2-x1))-log(h1)));
        end
       
     	Fl_h2 = lg*tmpH*(Ps+bern+pois+inst)+ lg*Psupra*(x2-xs)/(x2-x1)*((x2+xs)/2-x1);
        
    end
elseif (h1>0)&&(h2<=0)    %ajout pas NR (04/04/2007), pour calculer les forces de pression
    Fr_h1 = 0.5*(x2-x1)*lg*Ps; %en cas de fermeture au niveau d'une masse ou l'autre
    Fl_h2 = 0.5*(x2-x1)*lg*Ps; %ici, fermeture au niveau de la masse 2
elseif (h1<=0)&&(h2>0)
    Fr_h1 = 0.5*(x2-x1)*lg*Psupra; %ici fermeture au niveau de la masse 1
    Fl_h2 = 0.5*(x2-x1)*lg*Psupra;
end
 %modif par NR et CBV, sortie de cette ligne de la condition if de d�part
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%% Calcul de Fr_h2 %%%%%%%%%%%%%%%%%%%%%

     Fr_h2 = 0.5*lg*Psupra*(x3-x2); 

     Fvf = [ Fl_h1, Fl_h2; Fr_h1, Fr_h2 ];
     
end

function W = Wv1(xi,xip1,A,B,i)

%****************************************************%
% Calcul de la primitive de 1/h2  , entre xi et x(i+1), pour 
% la plaque numero i
% 
% Parametres d'entree
% 
% xi,xip1: abscisse entre lesquelles on calcule l'integrale
% A,B: coefficient de l'equation du la plaque,
% i: numero de la plaque
% 
% Parametre de sortie
% 
% X: valeur de l'integrale
%****************************************************%                                
% Formules de CV   
%****************************************************%

pres = 1e-5;

hi = A(i)*xi + B(i);  % = hi 
hip1 = A(i)*xip1 + B(i);    % = hi+1 
% 
% if (hi == hip1)
%     W = (xip1-xi)/(hi^2); %cas oe la plaque est horizontale
if (abs(A(i)*xip1/B(i))<pres) %cas de presque horizontalite, avec dvpt limite
      W2 = 1/B(i)^2*(xip1-A(i)*xip1^2/B(i)+A(i)^2*xip1^3/B(i)^2);
      W1 = 1/B(i)^2*(xi-A(i)*xi^2/B(i)+A(i)^2*xi^3/B(i)^2); 
      W = W2-W1;
else
    W = 1/A(i)*(1/hi-1/hip1);
end

end

function X = Xv1(xi,xip1,A,B,i)

%****************************************************%
% Calcul de la primitive de x/h2 , entre xi et x(i+1), pour 
% la plaque numero i
% 
% Parametres d'entree
% 
% xi,xip1: abscisse entre lesquelles on calcule l'integrale
% A,B: coefficient de l'equation du la plaque,
% i: numero de la plaque
% 
% Parametre de sortie
% 
% X: valeur de l'integrale
%****************************************************%

pres = 1e-5;
hi = A(i)*xi+B(i);  % = hi 
hip1 = A(i)*xip1+B(i);    % = hi+1 

% if (hi == hip1)
%     X = (xip1-xi)*xi/(hi^2) ; %cas oe la plaque est horizontale

if (abs(A(i)*xip1/B(i))<pres) %cas de presque horizontalite, avec dvpt limite
      X2 = 1/B(i)^2*(xip1^2/2-2*A(i)*xip1^3/3/B(i)+3*A(i)^2*xip1^4/4/B(i)^2);
      X1 = 1/B(i)^2*(xi^2/2-2*A(i)*xi^3/3/B(i)+3*A(i)^2*xi^4/4/B(i)^2); 
      X=X2-X1;
else
    X = 1/(A(i)^2)*log(hip1/hi)+1/A(i)*(xi-hi/A(i))*(1/hi-1/hip1);
end 

end

function Y = Yv1(xi,xip1,A,B,i)

%****************************************************%
% Calcul de la primitive de ln(h)  , entre xi et x(i+1), pour 
% la plaque numero i
% 
% Parametres d'entree
% 
% xi,xip1: abscisse entre lesquelles on calcule l'integrale
% A,B: coefficient de l'equation du la plaque,
% i: numero de la plaque
% 
% Parametre de sortie
% 
% X: valeur de l'integrale                              
%****************************************************%

pres = 1e-5;
hi = A(i)*xi+B(i);  % = hi 
hip1 = A(i)*xip1+B(i);    % = hi+1 
% if (hi == hip1)
%     Y = (xip1-xi)*log(hi) ; %cas oe la plaque est horizontale
if (abs(A(i)*xip1/B(i))<pres) %cas de presque horizontalite, avec dvpt limite
    Y2=(xip1*log(B(i))+A(i)/B(i)/2*xip1^2-A(i)^2/B(i)^2/6*xip1^3);
    Y1=(xi*log(B(i))+A(i)/B(i)/2*xi^2-A(i)^2/B(i)^2/6*xi^3);
    Y=Y1-Y2;
else
    Y = 1/A(i)*(hip1*log(hip1)-hip1-hi*log(hi)+hi);
end 

end

function Z = Zv1(xi,xip1,A,B,i)

%****************************************************%
% Calcul de la primitive de x*ln(h)    , entre xi et x(i+1), pour 
% la plaque numero i
% 
% Parametres d'entree
% 
% xi,xip1: abscisse entre lesquelles on calcule l'integrale
% A,B: coefficient de l'equation du la plaque,
% i: numero de la plaque
% 
% Parametre de sortie
% 
% X: valeur de l'integrale                            
%****************************************************%

pres = 1e-5;
hi = A(i)*xi+B(i);  % = hi 
hip1 = A(i)*xip1+B(i);    % = hi+1 
% if (hi == hip1)
%     Z = xi*(xip1-xi)*log(hi) ; %cas oe la plaque est horizontale
if (abs(A(i)*xip1/B(i))<pres) %cas de presque horizontalite, avec dvpt limite
    Z2=(xip1^2/2*log(B(i))+A(i)*xip1^3/B(i)/3-A(i)^2*xip1^4/8/B(i)^2);
    Z1=(xi^2/2*log(B(i))+A(i)*xi^3/B(i)/3-A(i)^2*xi^4/8/B(i)^2);
    Z=Z2-Z1;
else
    tempo = 1/(A(i)^2);
    tempo = tempo*(((hip1^2)*log(hip1)-(hi^2)*log(hi))/2-((hip1^2)-(hi^2))/4);
    Z     = tempo+1/A(i)*(xi-hi/A(i))*(hip1*log(hip1)-hip1-hi*log(hi)+hi);
    
end  

end

function [ yt, ytm1, ytm2 ] = displace2mass( Fvf, ytm1, ytm2, fs, mvf, ke, re, kc )

%%%%%%%%%%%%%% Resolution des equas mecaniques %%%%%%%%%%%%%%
%
% param�tres d'entr�e
% 
% Fl_h2,Fr_h2,Fl_h1,Fr_h1: forces de pression,
% y1uim1,y2uim1: positions des masses de la corde du haut � t-1
% y1uim2,y2uim2: positions des masses de la corde du haut � t-2
% y1dim1,y2dim1:positions des masses de la corde du bas � t-1
% y1dim2,y2dim2:positions des masses de la corde du bas � t-2
% fe: fr�quence d'�chantillonage
% m1u,m1d,m2u,m2d: masses
% k1de,k1ue,k2de,k2ue: raideurs associ�es aux masses
% r1ue,r1de,r2ue,r2de: amortissements associ�es aux masses
% kcd,kcu: raideurs de couplages des masses

%***************** Forces de pressions externes************%
F1u = sum(Fvf(:,1)); F2u = sum(Fvf(:,2)); F1d = F1u; F2d = F2u;
kcup = kc(1); kcdown = kc(2);

%******* Calcul des nouvelles positions  *****%
tmp1_A = mvf(1,1)*fs^2+ke(1,1)+re(1,1)*fs+kc(1);
tmp1_B = mvf(1,1)*fs^2*(2*ytm1(1,1)-ytm2(1,1))+re(1,1)*fs*ytm1(1,1);
tmp2_A = mvf(1,2)*fs^2+ke(1,2)+re(1,2)*fs+kc(1);
tmp2_B = mvf(1,2)*fs^2*(2*ytm1(1,2)-ytm2(1,2))+re(1,2)*fs*ytm1(1,2);

%******* Resolution systeme couple  ***********%
yt(1,1) = (F1u+tmp1_B+(kcup/tmp2_A)*(F2u+tmp2_B))/(tmp1_A-(kcup^2)/tmp2_A);
yt(1,2) = (F2u+tmp2_B+(kcup/tmp1_A)*(F1u+tmp1_B))/(tmp2_A-(kcup^2)/tmp1_A);

tmp1_A = mvf(2,1)*fs^2+ke(2,1)+re(2,1)*fs+kc(2);
tmp1_B = mvf(2,1)*fs^2*(2*ytm1(2,1)-ytm2(2,1))+re(2,1)*fs*ytm1(2,1);
tmp2_A = mvf(2,2)*fs^2+ke(2,2)+re(2,2)*fs+kc(2);
tmp2_B = mvf(2,2)*fs^2*(2*ytm1(2,2)-ytm2(2,2))+re(2,2)*fs*ytm1(2,2);
   
%******* Resolution systeme couple  ***********%
yt(2,1) = (F1d+tmp1_B+(kcdown/tmp2_A)*(F2d+tmp2_B))/(tmp1_A-(kcdown^2)/tmp2_A);
yt(2,2) = (F2d+tmp2_B+(kcdown/tmp1_A)*(F1d+tmp1_B))/(tmp2_A-(kcdown^2)/tmp1_A);
ytm2 = ytm1;
ytm1 = yt;

end
